<?php
/**
 * @version SOFORT iDEAL - $Date: 2013-06-12 16:55:41 +0200 (Wed, 12 Jun 2013) $
 * @author SOFORT AG (integration@sofort.com)
 * @link http://www.sofort.com/
 *
 * Copyright (c) 2012 SOFORT AG
 *
 * Released under the GNU General Public License (Version 2)
 * [http://www.gnu.org/licenses/gpl-2.0.html]
 *
 * $Id: sofort_ideal.php 6216 2013-06-12 14:55:41Z boehm $
 */

require_once(DIR_FS_CATALOG.'ext/modules/payment/sofort/helperFunctions.php');
require_once(DIR_FS_CATALOG.'ext/modules/payment/sofort/library/sofortLib.php');
require_once(DIR_FS_CATALOG.'ext/modules/payment/sofort/library/sofortLib_ideal_classic.php');


class sofort_ideal {
	var $code, $title, $description, $enabled, $sofort, $order_status;
	
	function sofort_ideal () {
		global $order;
		
		$this->code = 'sofort_ideal';
		$this->_checkExistingSofortConstants();
		$this->title = SOFORT_IDEAL_CLASSIC_TEXT_TITLE; //admin-view
 		$this->public_title = SOFORT_IDEAL_TEXT_TITLE; //public-view
		
		if (defined('SOFORT_IDEAL_CLASSIC_RECOMMENDED_PAYMENT') && SOFORT_IDEAL_CLASSIC_RECOMMENDED_PAYMENT == 'True') $this->public_title .= ' '.SOFORT_IDEAL_CLASSIC_RECOMMENDED_PAYMENT_TEXT;
		
		$this->description = SOFORT_IDEAL_CLASSIC_TEXT_DESCRIPTION.'<br />'.SOFORT_MULTIPAY_VERSIONNUMBER.': '.HelperFunctions::getSofortmodulVersion();
		$this->sort_order = SOFORT_IDEAL_CLASSIC_SORT_ORDER;
		$this->enabled = ((SOFORT_IDEAL_CLASSIC_STATUS == 'True') ? true : false);
		$this->order_status = SOFORT_IDEAL_CLASSIC_TMP_STATUS_ID;
		$this->icons_available = '';
		$this->email_footer = SOFORT_IDEAL_CLASSIC_TEXT_EMAIL_FOOTER;
		
		if ((int) SOFORT_IDEAL_CLASSIC_ORDER_STATUS_ID > 0) $this->order_status = SOFORT_IDEAL_CLASSIC_ORDER_STATUS_ID;
		
		if (is_object($order)) $this->update_status();
		
		//$this->defaultCurrency = DEFAULT_CURRENCY;
		$this->sofort = new SofortLib_iDealClassic (SOFORT_IDEAL_CLASSIC_CONFIGURATION_KEY, SOFORT_IDEAL_CLASSIC_PROJECT_PASSWORD, 'sha1');
		$this->sofort->setVersion(HelperFunctions::getSofortmodulVersion());
		$this->form_action_url = DIR_WS_CATALOG . 'ext/modules/payment/sofort/ressources/scripts/redirect.php';
	}
	
	
	/**
	 * This function outputs the payment method title/text and if required, the input fields when user has finished shopping
	 */
	function selection () {
		//check, if an aborted order is saved and must be set "aborted"
		$this->_checkCancelOrder();
		
		$text = '';
		
		switch (SOFORT_IDEAL_CLASSIC_IMAGE) {
			case 'Logo & Text': $text .= $this->_setImageText('logo_155x50.png', SOFORT_IDEAL_CLASSIC_TEXT_DESCRIPTION_CHECKOUT_PAYMENT_TEXT); break;
			case 'Infographic':
			default:			$text .= $this->_setImageText('banner_300x100.png', ''); break;
		}
		
		//get all available banks from SOFORT-server
		$banks = $this->sofort->getRelatedBanks();
		
		//create <select> - usage of tep_draw_pull_down_menu() is more code than following lines
		$selectBox = '<select name="ideal_bank_name" size="1"><option value="0">---</option>';
		
		if(!empty($banks) && is_array($banks)) {
			foreach ($banks as $value) {
				$selectBox .= '<option value="'.$value['code'].'"';
				
				if( isset( $_SESSION[ 'ideal_bank_name' ] ) && $_SESSION['ideal_bank_name'] == $value['code']) $selectBox .= ' selected="selected"';
				
				$selectBox .= '>'.$value['name'].'</option>';
			}
		}
		
		$selectBox .= '</select>';
		$title = str_replace('{{selectbox}}', $selectBox, $text);
		$fields = array(
				array(
					'title' => $title,
				)
		);
		
		return array('id' => $this->code , 'module' => $this->public_title, 'fields' => $fields);
	}
	
	
	function pre_confirmation_check () {
		$requestData = $_POST;
		
		if(isset($requestData['ideal_bank_name'])) $_SESSION['ideal_bank_name'] = HelperFunctions::htmlMask($requestData['ideal_bank_name']);
		
		if ((!isset($requestData['ideal_bank_name']) || !$requestData['ideal_bank_name']) && (!isset($_SESSION['ideal_bank_name']) || !$_SESSION['ideal_bank_name'])) {
				$paymentError = 'payment_error='.$this->code.'&error_codes=10000';
				$returnLink = tep_href_link(FILENAME_CHECKOUT_PAYMENT, $paymentError, 'SSL', true, false);
				tep_redirect(HelperFunctions::cleanUrlParameter($returnLink));
		}
		
		return false;
	}
	
	
	function install () {
		$sofortStatuses = $this->_insertAndReturnSofortStatus();
		$tempStatus = (isset($sofortStatuses['ideal_temp'])&& !empty($sofortStatuses['ideal_temp']))? $sofortStatuses['ideal_temp'] : '';
		$abortedStatus = (isset($sofortStatuses['ideal_aborted'])&& !empty($sofortStatuses['ideal_aborted']))? $sofortStatuses['ideal_aborted'] : '';
		$confirmedStatus = (isset($sofortStatuses['ideal_confirmed'])&& !empty($sofortStatuses['ideal_confirmed']))? $sofortStatuses['ideal_confirmed'] : '';
		$idealLossStatus = (isset($sofortStatuses['ideal_loss'])&& !empty($sofortStatuses['ideal_loss']))? $sofortStatuses['ideal_loss'] : '';
		$idealRefRefStatus = (isset($sofortStatuses['ideal_ref_ref'])&& !empty($sofortStatuses['ideal_ref_ref']))? $sofortStatuses['ideal_ref_ref'] : '';
		$idealRefComStatus = (isset($sofortStatuses['ideal_ref_com'])&& !empty($sofortStatuses['ideal_ref_com']))? $sofortStatuses['ideal_ref_com'] : '';
		
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value, configuration_group_id, sort_order, set_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_STATUS_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_STATUS_DESC)."', 'SOFORT_IDEAL_CLASSIC_STATUS', 'False', '6', '1', 'tep_cfg_select_option(array(\'True\', \'False\'), ', now())");
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value, configuration_group_id, sort_order, set_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_RECOMMENDED_PAYMENT_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_RECOMMENDED_PAYMENT_DESC)."', 'SOFORT_IDEAL_CLASSIC_RECOMMENDED_PAYMENT', 'False', '6', '2', 'tep_cfg_select_option(array(\'True\', \'False\'), ', now())");
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value, configuration_group_id, sort_order, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_CONFIGURATION_KEY_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_CONFIGURATION_KEY_DESC)."', 'SOFORT_IDEAL_CLASSIC_CONFIGURATION_KEY', '',  '6', '3', now())");
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value, configuration_group_id, sort_order, set_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_AUTH_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_AUTH_DESC)."', 'SOFORT_IDEAL_CLASSIC_AUTH', '---',  '6', '4', 'tep_cfg_select_option(array(),', now())");  //hide the input-field with an empty <select>
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value, configuration_group_id, sort_order, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_PROJECT_PASSWORD_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_PROJECT_PASSWORD_DESC)."', 'SOFORT_IDEAL_CLASSIC_PROJECT_PASSWORD', '',  '6', '4', now())");
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value, configuration_group_id, sort_order, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_NOTIFICATION_PASSWORD_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_NOTIFICATION_PASSWORD_DESC)."', 'SOFORT_IDEAL_CLASSIC_NOTIFICATION_PASSWORD', '',  '6', '5', now())");
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value, configuration_group_id, sort_order, use_function, set_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_ZONE_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_ZONE_DESC)."', 'SOFORT_IDEAL_CLASSIC_ZONE', '0', '6', '6', 'tep_get_zone_class_title', 'tep_cfg_pull_down_zone_classes(', now())");
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value, configuration_group_id, sort_order, set_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_REASON_1_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_REASON_1_DESC)."', 'SOFORT_IDEAL_CLASSIC_REASON_1', 'Nr. {{order_id}} Kd-Nr. {{customer_id}}',  '6', '7', 'tep_cfg_select_option(array(\'Nr. {{order_id}} Kd-Nr. {{customer_id}}\'), ', now())");
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value, configuration_group_id, sort_order, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_ALLOWED_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_ALLOWED_DESC)."', 'SOFORT_IDEAL_ALLOWED', 'NL',  '6', '8', now())");
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value, configuration_group_id, sort_order, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_REASON_2_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_REASON_2_DESC)."', 'SOFORT_IDEAL_CLASSIC_REASON_2', '', '6', '9', now())");
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value, configuration_group_id, sort_order, set_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_IMAGE_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_IMAGE_DESC)."', 'SOFORT_IDEAL_CLASSIC_IMAGE', 'Infographic',  '6', '10', 'tep_cfg_select_option(array(\'Infographic\',\'Logo & Text\'), ', now())");
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value, configuration_group_id, sort_order, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_SORT_ORDER_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_SORT_ORDER_DESC)."', 'SOFORT_IDEAL_CLASSIC_SORT_ORDER', '0', '6', '14', now())");
		
		//abort-status: set if buyer cancels the wizard (Warning: in about 2%-3% of all live-transactions, this status is not set correct! (Problem: iDEAL is not fast enough and SOFORT gets wrong status from iDEAL)
		//this may cause unusual history-entries but never the "paid"-status!
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value, configuration_group_id, sort_order, set_function, use_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_CANCELED_ORDER_STATUS_ID_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_CANCELED_ORDER_STATUS_ID_DESC)."', 'SOFORT_IDEAL_CLASSIC_CANCELED_ORDER_STATUS_ID', '".HelperFunctions::escapeSql($abortedStatus)."',  '6', '13', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
		//Temp-Status - set while checkout and also set with notification pending-not_credited_yet
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value, configuration_group_id, sort_order, set_function, use_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_TMP_STATUS_ID_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_TMP_STATUS_ID_DESC)."', 'SOFORT_IDEAL_CLASSIC_TMP_STATUS_ID', '".HelperFunctions::escapeSql($tempStatus)."',  '6', '11', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
		//Paid-Status - received_credited
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value, configuration_group_id, sort_order, set_function, use_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_ORDER_STATUS_ID_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_ORDER_STATUS_ID_DESC)."', 'SOFORT_IDEAL_CLASSIC_ORDER_STATUS_ID', '".HelperFunctions::escapeSql($confirmedStatus)."',  '6', '12', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
		//Buyer did not pay - loss-not_credited
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value,  configuration_group_id, sort_order, set_function, use_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_LOS_NOT_CRE_STATUS_ID_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_LOS_NOT_CRE_STATUS_ID_DESC)."', 'SOFORT_IDEAL_CLASSIC_LOS_NOT_CRE_STATUS_ID', '".HelperFunctions::escapeSql($idealLossStatus)."',  '6', '30', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
		//partically refunded - refunded-compensation
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value,  configuration_group_id, sort_order, set_function, use_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_REF_COM_STATUS_ID_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_REF_COM_STATUS_ID_DESC)."', 'SOFORT_IDEAL_CLASSIC_REF_COM_STATUS_ID', '".HelperFunctions::escapeSql($idealRefComStatus)."',  '6', '30', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
		//refunded by seller - refunded-refunded
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value,  configuration_group_id, sort_order, set_function, use_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_REF_REF_STATUS_ID_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_REF_REF_STATUS_ID_DESC)."', 'SOFORT_IDEAL_CLASSIC_REF_REF_STATUS_ID', '".HelperFunctions::escapeSql($idealRefRefStatus)."',  '6', '30', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
		//automatically(!) refunded - late_succeed-automatic_refund_to_customer
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value,  configuration_group_id, sort_order, set_function, use_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_LAT_SUC_AUT_STATUS_ID_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_LAT_SUC_AUT_STATUS_ID_DESC)."', 'SOFORT_IDEAL_CLASSIC_LAT_SUC_AUT_STATUS_ID', '".HelperFunctions::escapeSql($idealRefRefStatus)."',  '6', '30', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
	}
	
	
	function remove () {
		tep_db_query("DELETE FROM ".TABLE_CONFIGURATION." WHERE configuration_key LIKE 'SOFORT_IDEAL_%'");
	}
	
	
	function keys () {
		return array(
			'SOFORT_IDEAL_CLASSIC_STATUS',
			'SOFORT_IDEAL_CLASSIC_RECOMMENDED_PAYMENT',
			'SOFORT_IDEAL_CLASSIC_CONFIGURATION_KEY',
			'SOFORT_IDEAL_CLASSIC_AUTH',
			'SOFORT_IDEAL_CLASSIC_PROJECT_PASSWORD',
			'SOFORT_IDEAL_CLASSIC_NOTIFICATION_PASSWORD',
			'SOFORT_IDEAL_CLASSIC_ZONE',
			'SOFORT_IDEAL_ALLOWED',
			'SOFORT_IDEAL_CLASSIC_REASON_1',
			'SOFORT_IDEAL_CLASSIC_REASON_2',
			'SOFORT_IDEAL_CLASSIC_IMAGE' ,
			'SOFORT_IDEAL_CLASSIC_TMP_STATUS_ID' ,
			'SOFORT_IDEAL_CLASSIC_ORDER_STATUS_ID',
			'SOFORT_IDEAL_CLASSIC_CANCELED_ORDER_STATUS_ID',
			'SOFORT_IDEAL_CLASSIC_LOS_NOT_CRE_STATUS_ID',
			'SOFORT_IDEAL_CLASSIC_REF_COM_STATUS_ID',
			'SOFORT_IDEAL_CLASSIC_REF_REF_STATUS_ID',
			'SOFORT_IDEAL_CLASSIC_LAT_SUC_AUT_STATUS_ID',
			'SOFORT_IDEAL_CLASSIC_SORT_ORDER'
		);
	}
	
	
	function update_status () {
		global $order;
		
		if (($this->enabled == true) && ((int) SOFORT_IDEAL_CLASSIC_ZONE > 0)) {
			$check_flag = false;
			$check_query = tep_db_query("SELECT	  zone_id
										 FROM	  ".TABLE_ZONES_TO_GEO_ZONES."
										 WHERE	  geo_zone_id = '".HelperFunctions::escapeSql(SOFORT_IDEAL_CLASSIC_ZONE)."'
										 AND	  zone_country_id = '".HelperFunctions::escapeSql($order->billing['country']['id'])."'
										 ORDER BY zone_id");
			
			while ($check = tep_db_fetch_array($check_query)) {
				if ($check['zone_id'] < 1 || $check['zone_id'] == $order->billing['zone_id']) {
					$check_flag = true;
					break;
				}
			}
			
			if ($check_flag == false) $this->enabled = false;
		}
	}
	
	
	function javascript_validation () {
		return false;
	}
	
	
	function _setImageText($image, $text) {
		$lng = HelperFunctions::getShortCode($_SESSION['language']);
		$image = 'https://images.sofort.com/'.$lng.'/ideal/'.$image;
		$image = tep_image($image, SOFORT_IDEAL_CLASSIC_TEXT_DESCRIPTION_CHECKOUT_PAYMENT_IMAGEALT);
		$title = SOFORT_IDEAL_CLASSIC_TEXT_DESCRIPTION_CHECKOUT_PAYMENT_IMAGE;
		$title = str_replace('{{image}}', $image, $title);
		$title = str_replace('{{text}}', $text, $title);
		
		return $title;
	}
	
	
	function confirmation () {
		global $cartID, $cart_pn_ID, $customer_id, $languages_id, $language, $order, $order_total_modules, $redirect, $insert_id;
		
		//this will be displayed on confirmation page "payment information"
		$confirmation = array(
				'title' => SOFORT_IDEAL_CLASSIC_CHECKOUT_CONFIRMATION,
				'fields' => array(array())
		);
		
		//wait for redirect-page
		if($redirect !== true) return $confirmation;
		
		////////////////////////////////////////////////////////////////////////////////////////////////
		//following code inserts the order into shop-db. its called directly before forwarding to pnag//
		////////////////////////////////////////////////////////////////////////////////////////////////
		
		$order_totals = array();
		
		if (is_array($order_total_modules->modules)) {
			reset($order_total_modules->modules);
			
			while (list(, $value) = each($order_total_modules->modules)) {
				$class = substr($value, 0, strrpos($value, '.'));
				
				if ($GLOBALS[$class]->enabled) {
					for ($i=0, $n=sizeof($GLOBALS[$class]->output); $i<$n; $i++) {
						if (tep_not_null($GLOBALS[$class]->output[$i]['title']) && tep_not_null($GLOBALS[$class]->output[$i]['text'])) {
							$order_totals[] = array(
									'code'		 => $GLOBALS[$class]->code,
									'title'		 => $GLOBALS[$class]->output[$i]['title'],
									'text'		 => $GLOBALS[$class]->output[$i]['text'],
									'value'		 => $GLOBALS[$class]->output[$i]['value'],
									'sort_order' => $GLOBALS[$class]->sort_order,
							);
						}
					}
				}
			}
		}
		
		$sql_data_array = array(
				'customers_id'				  => $customer_id,
				'customers_name'			  => $order->customer['firstname'].' '.$order->customer['lastname'],
				'customers_company'			  => $order->customer['company'],
				'customers_street_address'	  => $order->customer['street_address'],
				'customers_suburb'			  => $order->customer['suburb'],
				'customers_city'			  => $order->customer['city'],
				'customers_postcode'		  => $order->customer['postcode'],
				'customers_state'			  => $order->customer['state'],
				'customers_country'			  => $order->customer['country']['title'],
				'customers_telephone'		  => $order->customer['telephone'],
				'customers_email_address'	  => $order->customer['email_address'],
				'customers_address_format_id' => $order->customer['format_id'],
				'delivery_name'				  => $order->delivery['firstname'].' '.$order->delivery['lastname'],
				'delivery_company'			  => $order->delivery['company'],
				'delivery_street_address'	  => $order->delivery['street_address'],
				'delivery_suburb'			  => $order->delivery['suburb'],
				'delivery_city'				  => $order->delivery['city'],
				'delivery_postcode'			  => $order->delivery['postcode'],
				'delivery_state'			  => $order->delivery['state'],
				'delivery_country'			  => $order->delivery['country']['title'],
				'delivery_address_format_id'  => $order->delivery['format_id'],
				'billing_name'				  => $order->billing['firstname'].' '.$order->billing['lastname'],
				'billing_company'			  => $order->billing['company'],
				'billing_street_address'	  => $order->billing['street_address'],
				'billing_suburb'			  => $order->billing['suburb'],
				'billing_city'				  => $order->billing['city'],
				'billing_postcode'			  => $order->billing['postcode'],
				'billing_state'				  => $order->billing['state'],
				'billing_country'			  => $order->billing['country']['title'],
				'billing_address_format_id'	  => $order->billing['format_id'],
				'payment_method'			  => $order->info['payment_method'],
				'cc_type'					  => $order->info['cc_type'],
				'cc_owner'					  => $order->info['cc_owner'],
				'cc_number'					  => $order->info['cc_number'],
				'cc_expires'				  => $order->info['cc_expires'],
				'orders_status'				  => SOFORT_IDEAL_CLASSIC_TMP_STATUS_ID,
				'currency'					  => $order->info['currency'],
				'currency_value'			  => $order->info['currency_value'],
				'date_purchased'			  => 'now()',
		);
		tep_db_perform(TABLE_ORDERS, $sql_data_array);
		$insert_id = tep_db_insert_id();
		
		for ($i=0, $n=sizeof($order_totals); $i<$n; $i++) {
			$sql_data_array = array(
					'orders_id'	 => $insert_id,
					'title'		 => $order_totals[$i]['title'],
					'text'		 => $order_totals[$i]['text'],
					'value'		 => $order_totals[$i]['value'],
					'class'		 => $order_totals[$i]['code'],
					'sort_order' => $order_totals[$i]['sort_order'],
			);
			tep_db_perform(TABLE_ORDERS_TOTAL, $sql_data_array);
		}
		
		for ($i=0, $n=sizeof($order->products); $i<$n; $i++) {
			$sql_data_array = array(
					'orders_id'			=> $insert_id,
					'products_id'		=> tep_get_prid($order->products[$i]['id']),
					'products_model'	=> $order->products[$i]['model'],
					'products_name'		=> $order->products[$i]['name'],
					'products_price'	=> $order->products[$i]['price'],
					'final_price'		=> $order->products[$i]['final_price'],
					'products_tax'		=> $order->products[$i]['tax'],
					'products_quantity' => $order->products[$i]['qty'],
			);
			tep_db_perform(TABLE_ORDERS_PRODUCTS, $sql_data_array);
			$order_products_id = tep_db_insert_id();
			$attributes_exist = '0';
				
			if (isset($order->products[$i]['attributes'])) {
				$attributes_exist = '1';
				
				for ($j=0, $n2=sizeof($order->products[$i]['attributes']); $j<$n2; $j++) {
					if (DOWNLOAD_ENABLED == 'true') {
						$attributes_query = "SELECT		popt.products_options_name		  ,
														poval.products_options_values_name,
														pa.options_values_price			  ,
														pa.price_prefix					  ,
														pad.products_attributes_maxdays	  ,
														pad.products_attributes_maxcount  ,
														pad.products_attributes_filename
											FROM		".TABLE_PRODUCTS_OPTIONS." popt		   ,
														".TABLE_PRODUCTS_OPTIONS_VALUES." poval,
														".TABLE_PRODUCTS_ATTRIBUTES." pa
											LEFT JOIN	".TABLE_PRODUCTS_ATTRIBUTES_DOWNLOAD." pad
											ON			pa.products_attributes_id=pad.products_attributes_id
											WHERE pa.products_id = '" . HelperFunctions::escapeSql($order->products[$i]['id']) . "'
											AND			pa.options_id = '".HelperFunctions::escapeSql($order->products[$i]['attributes'][$j]['option_id'])."'
											AND			pa.options_id = popt.products_options_id
											AND			pa.options_values_id = '".HelperFunctions::escapeSql($order->products[$i]['attributes'][$j]['value_id'])."'
											AND			pa.options_values_id = poval.products_options_values_id
											AND			popt.language_id = '".HelperFunctions::escapeSql($languages_id)."'
											AND			poval.language_id = '".HelperFunctions::escapeSql($languages_id)."'
						";
						$attributes = tep_db_query($attributes_query);
					} else {
						$attributes = tep_db_query("SELECT	popt.products_options_name		  ,
															poval.products_options_values_name,
															pa.options_values_price			  ,
															pa.price_prefix
													FROM	".TABLE_PRODUCTS_OPTIONS." popt		   ,
															".TABLE_PRODUCTS_OPTIONS_VALUES." poval,
															".TABLE_PRODUCTS_ATTRIBUTES." pa
													WHERE	pa.products_id = '".HelperFunctions::escapeSql($order->products[$i]['id'])."'
													AND		pa.options_id = '".HelperFunctions::escapeSql($order->products[$i]['attributes'][$j]['option_id'])."'
													AND		pa.options_id = popt.products_options_id
													AND		pa.options_values_id = '".HelperFunctions::escapeSql($order->products[$i]['attributes'][$j]['value_id'])."'
													AND		pa.options_values_id = poval.products_options_values_id
													AND		popt.language_id = '".HelperFunctions::escapeSql($languages_id)."'
													AND		poval.language_id = '".HelperFunctions::escapeSql($languages_id)."'
						");
					}
					
					$attributes_values = tep_db_fetch_array($attributes);
					$sql_data_array = array(
							'orders_id'				  => $insert_id,
							'orders_products_id'	  => $order_products_id,
							'products_options'		  => $attributes_values['products_options_name'],
							'products_options_values' => $attributes_values['products_options_values_name'],
							'options_values_price'	  => $attributes_values['options_values_price'],
							'price_prefix'			  => $attributes_values['price_prefix'],
					);
					tep_db_perform(TABLE_ORDERS_PRODUCTS_ATTRIBUTES, $sql_data_array);
					
					if ((DOWNLOAD_ENABLED == 'true') && isset($attributes_values['products_attributes_filename']) && tep_not_null($attributes_values['products_attributes_filename'])) {
						$sql_data_array = array(
								'orders_id'				   => $insert_id,
								'orders_products_id'	   => $order_products_id,
								'orders_products_filename' => $attributes_values['products_attributes_filename'],
								'download_maxdays'		   => $attributes_values['products_attributes_maxdays'],
								'download_count'		   => $attributes_values['products_attributes_maxcount'],
						);
						tep_db_perform(TABLE_ORDERS_PRODUCTS_DOWNLOAD, $sql_data_array);
					}
				}
			}
		}
		
		$cart_pn_ID = $cartID.'-'.$insert_id;
		tep_session_register('cart_pn_ID');
		
		return true;
	}
	
	
	function before_process () {
		global $customer_id, $order, $order_totals, $sendto, $billto, $languages_id, $payment, $currencies, $currency, $cart, $cart_pn_ID, $$payment;
		
		//fix bug in < osc 2.2 RC2
		if(!is_array($order_totals)){
			include(DIR_WS_CLASSES.'order_total.php');
			$order_total_modules = new order_total;
			$order_totals = $order_total_modules->process();
		}
		
		$order_id = substr($cart_pn_ID, strpos($cart_pn_ID, '-')+1);
		$tmpStatusId = SOFORT_IDEAL_CLASSIC_TMP_STATUS_ID;
		
		if(!empty($order->info['comments'])){
			$sql_data_array = array(
					'orders_id'			=> $order_id,
					'orders_status_id'	=> $tmpStatusId,
					'date_added'		=> 'now()',
					'customer_notified'	=> (SEND_EMAILS == 'true') ? '1' : '0',
					'comments'			=> $order->info['comments']);
			tep_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);
		}
		
		$order_totals_table_top = '<table border="0" cellpadding="5" cellspacing="0">';
		$order_totals_cell_top = '<tr><td width="280" style="font-size: 12px">';
		$order_totals_cell_mid = '</td><td style="font-size: 12px" align="right">';
		$order_totals_cell_end = '</td></tr>';
		$order_totals_table_end = '</table>';
		
		// initialized for the email confirmation
		$products_ordered = (EMAIL_USE_HTML == 'true') ? $order_totals_table_top : '';
		
		$subtotal = 0;
		$total_tax = 0;
		
		for ($i=0, $n=sizeof($order->products); $i<$n; $i++) {
			// Update products_ordered (for bestsellers list)
			tep_db_query("UPDATE ".TABLE_PRODUCTS." SET products_ordered = products_ordered + ".sprintf('%d', $order->products[$i]['qty'])." WHERE products_id = '".tep_get_prid(HelperFunctions::escapeSql($order->products[$i]['id']))."'");
				
			//insert customer chosen option to order
			$attributes_exist = '0';
			$products_ordered_attributes = '';
				
			if (isset($order->products[$i]['attributes'])) {
				$attributes_exist = '1';
				
				for ($j=0, $n2=sizeof($order->products[$i]['attributes']); $j<$n2; $j++) {
					if (DOWNLOAD_ENABLED == 'true') {
						$attributes_query = "SELECT		popt.products_options_name		  ,
														poval.products_options_values_name,
														pa.options_values_price			  ,
														pa.price_prefix					  ,
														pad.products_attributes_maxdays	  ,
														pad.products_attributes_maxcount  ,
														pad.products_attributes_filename
											FROM		".TABLE_PRODUCTS_OPTIONS." popt		   ,
														".TABLE_PRODUCTS_OPTIONS_VALUES." poval,
														".TABLE_PRODUCTS_ATTRIBUTES." pa
											LEFT JOIN	".TABLE_PRODUCTS_ATTRIBUTES_DOWNLOAD." pad
											ON			pa.products_attributes_id=pad.products_attributes_id
											WHERE		pa.products_id = '".HelperFunctions::escapeSql($order->products[$i]['id'])."'
											AND		pa.options_id = '".HelperFunctions::escapeSql($order->products[$i]['attributes'][$j]['option_id'])."'
											AND		pa.options_id = popt.products_options_id
											AND		pa.options_values_id = '".HelperFunctions::escapeSql($order->products[$i]['attributes'][$j]['value_id'])."'
											AND		pa.options_values_id = poval.products_options_values_id
											AND		popt.language_id = '".HelperFunctions::escapeSql($languages_id)."'
											AND		poval.language_id = '".HelperFunctions::escapeSql($languages_id)."'
						";
						$attributes = tep_db_query($attributes_query);
					} else {
						$attributes = tep_db_query("SELECT	popt.products_options_name,
															poval.products_options_values_name,
															pa.options_values_price,
															pa.price_prefix
													FROM	".TABLE_PRODUCTS_OPTIONS." popt,
															".TABLE_PRODUCTS_OPTIONS_VALUES." poval,
															".TABLE_PRODUCTS_ATTRIBUTES." pa
													WHERE 	pa.products_id = '".HelperFunctions::escapeSql($order->products[$i]['id'])."'
													AND		pa.options_id = '".HelperFunctions::escapeSql($order->products[$i]['attributes'][$j]['option_id'])."'
													AND		pa.options_id = popt.products_options_id and pa.options_values_id = '".HelperFunctions::escapeSql($order->products[$i]['attributes'][$j]['value_id'])."'
													AND 	pa.options_values_id = poval.products_options_values_id and popt.language_id = '".HelperFunctions::escapeSql($languages_id)."'
													AND 	poval.language_id = '".HelperFunctions::escapeSql($languages_id)."'
								");
					}
						
					$attributes_values = tep_db_fetch_array($attributes);
					$products_ordered_attributes .= "\n\t".$attributes_values['products_options_name'].' '.$attributes_values['products_options_values_name'];
				}
			}
				
			//------insert customer choosen option eof ----
			$total_weight += ($order->products[$i]['qty'] * $order->products[$i]['weight']);
			$total_tax += tep_calculate_tax($total_products_price, $products_tax) * $order->products[$i]['qty'];
			$total_cost += $total_products_price;
				
			if (EMAIL_USE_HTML == 'true'){
				$products_ordered .= $order_totals_cell_top;
				$products_ordered .= (int)$order->products[$i]['qty'].' x '.$this->cleanString($order->products[$i]['name']);
				$products_ordered .= ' ('.$this->cleanString($order->products[$i]['model']).') = ';
				$prodcuts_ordered .= $order_totals_cell_mid;
				$products_ordered .= $currencies->display_price($this->cleanString($order->products[$i]['final_price']),$this->cleanString($order->products[$i]['tax']),(int)$order->products[$i]['qty']).$products_ordered_attributes;
				$products_ordered .= $order_totals_cell_end;
			} else {
				$products_ordered .= (int)$order->products[$i]['qty'].' x '.$this->cleanString($order->products[$i]['name']);
				$products_ordered .= ' ('.$this->cleanString($order->products[$i]['model']).') = ';
				$products_ordered .= $currencies->display_price($this->cleanString($order->products[$i]['final_price']),$this->cleanString($order->products[$i]['tax']),(int)$order->products[$i]['qty']).$products_ordered_attributes."\n";
			}
		}
		
		if (EMAIL_USE_HTML == 'true'){
			$products_ordered .= $order_totals_table_end;
			$text = preg_replace('#([\n\t])|([ ]+)#', ' ', SOFORT_MULTIPAY_EMAIL_HTML_TEXT);
		} else {
			$text = SOFORT_MULTIPAY_EMAIL_TEXT;
		}
		
		$text = str_replace('{{ORDER_ID}}', $order_id, $text);
		$text = str_replace('{{INVOICE_URL}}', tep_href_link(FILENAME_ACCOUNT_HISTORY_INFO, 'order_id=' . $order_id, 'SSL', false), $text);
		$text = str_replace('{{DATE_ORDERED}}', strftime(DATE_FORMAT_LONG), $text);
		$text = ($order->info['comments']) ? str_replace('{{CUSTOMER_COMMENT}}', tep_db_output($order->info['comments']), $text) : str_replace('{{CUSTOMER_COMMENT}}', '', $text);
		$text = str_replace('{{Item_List}}', $products_ordered, $text);
		
		if (EMAIL_USE_HTML == 'true'){
			$list_total = $order_totals_table_top;
				
			for ($i=0, $n=sizeof($order_totals); $i<$n; $i++) {
				$list_total .= $order_totals_cell_top.$this->cleanString($order_totals[$i]['title']).$order_totals_cell_mid.$this->cleanString($order_totals[$i]['text']).$order_totals_cell_end;
			}
				
			$list_total .= $order_totals_table_end;
		} else {
			for ($i=0, $n=sizeof($order_totals); $i<$n; $i++) {
				$list_total .= $this->cleanString($order_totals[$i]['title']).' '.$this->cleanString($order_totals[$i]['text'])."\n";
			}
		}
		
		$text = str_replace('{{List_Total}}', $list_total, $text);
		
		if ($order->content_type != 'virtual') {
			$text = str_replace('{{DELIVERY_ADRESS}}', tep_address_label($customer_id, $sendto, 0, '', "\n"), $text);
		} elseif ($order->content_type == 'virtual') {
			if ((DOWNLOAD_ENABLED == 'true') && isset($attributes_values['products_attributes_filename']) && tep_not_null($attributes_values['products_attributes_filename'])) {
				$text = str_replace('{{DELIVERY_ADRESS}}', tep_href_link(FILENAME_ACCOUNT_HISTORY_INFO, 'order_id=' . $order_id, 'SSL', false), $text);
			} else {
				$text = str_replace('{{DELIVERY_ADRESS}}', '', $text);
			}
		} else {
			$text = str_replace('{{DELIVERY_ADRESS}}', '', $text);
		}
		
		$text = str_replace('{{BILLING_ADRESS}}', tep_address_label($customer_id, $billto, 0, '', "\n"), $text);
		
		if (is_object($$payment)) {
			$payment_class = $$payment;
			$payment_text = $payment_class->public_title;
			
			$text = str_replace('{{Payment_Modul_Text}}', $payment_text, $text);
			$payment_email_footer = ''.$payment_class->email_footer;
			$text = str_replace('{{Payment_Modul_Text_Footer}}', $payment_email_footer, $text);
		}
		
		//replace all {{STORE_*}} {{OWNER_*}} {{EMAIL_*}} with defines
		//STORE_NAME STORE_NAME_ADDRESS OWNER_BANK_FA OWNER_BANK_TAX_NUMBER OWNER_BANK_UST_NUMBER OWNER_BANK_NAME OWNER_BANK_ACCOUNT STORE_OWNER_BLZ OWNER_BANK OWNER_BANK_SWIFT OWNER_BANK_IBAN
		$text = preg_replace("#\{\{((STORE_|OWNER_|EMAIL_)[A-Z0-9_]+)\}\}#e", '$1', $text);
		$email_order = $text;
		$email_subject = SOFORT_MULTIPAY_TEXT_EMAIL_SUBJECT;
		$email_subject = preg_replace("#\{\{((STORE_|OWNER_|EMAIL_)[A-Z0-9_]+)\}\}#e", '$1', $email_subject);
		$email_subject = str_replace('{{ORDER_ID}}', $order_id, $email_subject);
		$email_subject = str_replace('{{DATE_ORDERED}}', strftime(DATE_FORMAT_LONG), $email_subject);
		
		if(SEND_EMAILS == 'true' && $order_id) {
			tep_mail($this->cleanString($order->customer['firstname']).' '.$this->cleanString($order->customer['lastname']), $this->cleanString($order->customer['email_address']),$this->cleanString($email_subject),$email_order,STORE_OWNER, STORE_OWNER_EMAIL_ADDRESS);
			
			//set the customer-notified-flag
			tep_db_query("UPDATE ".TABLE_ORDERS_STATUS_HISTORY." SET customer_notified='1' WHERE orders_id='".(int)$order_id."' ORDER BY orders_status_history_id ASC LIMIT 1");
			
		}
		
		// send emails to other people
		if (SEND_EXTRA_ORDER_EMAILS_TO != '') {
			tep_mail('', SEND_EXTRA_ORDER_EMAILS_TO, $email_subject , $email_order, STORE_OWNER, STORE_OWNER_EMAIL_ADDRESS);
		}
		
		// update order_status to the selected status
		tep_db_query("UPDATE ".TABLE_ORDERS." SET orders_status='".SOFORT_IDEAL_CLASSIC_ORDER_STATUS_ID."' WHERE orders_id='".(int)$order_id."'");
		$time = $time = date("d.m.Y, G:i:s");
		HelperFunctions::insertHistoryEntry((int)$order_id, SOFORT_IDEAL_CLASSIC_ORDER_STATUS_ID, '', '', $time);
		
		// load the after_process function from the payment modules
		$this->after_process();
		$cart->reset(true);
		
		// unregister session variables used during checkout
		tep_session_unregister('sendto');
		tep_session_unregister('billto');
		tep_session_unregister('shipping');
		tep_session_unregister('payment');
		tep_session_unregister('comments');
		tep_session_unregister('cart_pn_ID');
		
		tep_redirect(tep_href_link(FILENAME_CHECKOUT_SUCCESS, '', 'SSL'));
	}
	
	
	/**
	 * The corresponding order is being processed here, bought articles being credited
	 * SofortLib calls SOFORT API
	 */
	function process_button () {
		global $order, $insert_id, $shopEncoding, $redirect;
		
		if ($redirect === true) {
			$customer_id = $_SESSION['customer_id'];
			$order_id = $insert_id;
			$_SESSION['cart_pn_sofortueberweisung_ID'] = $_SESSION['cart']->cartID.'-'.$insert_id;
			
			$total = ($_SESSION['customers_status']['customers_status_show_price_tax'] == 0 && $_SESSION['customers_status']['customers_status_add_tax_ot'] == 1) ? $order->info['total'] + $order->info['tax'] : $order->info['total'];
			
			$currency = $_SESSION['currency'];
			$reason_1 = str_replace('{{order_id}}', $order_id, SOFORT_IDEAL_CLASSIC_REASON_1);
			$reason_1 = str_replace('{{customer_id}}', $customer_id, $reason_1);
			$reason_1 = substr($reason_1, 0, 27);
			$reason_2 = str_replace('{{order_id}}', $order_id, SOFORT_IDEAL_CLASSIC_REASON_2);
			$reason_2 = str_replace('{{customer_id}}', $customer_id, $reason_2);
			$reason_2 = str_replace('{{order_date}}', strftime(DATE_FORMAT_SHORT), $reason_2);
			$reason_2 = str_replace('{{customer_name}}', $order->customer['firstname'].' '.$order->customer['lastname'], $reason_2);
			$reason_2 = str_replace('{{customer_company}}', $order->customer['company'], $reason_2);
			$reason_2 = str_replace('{{customer_email}}', $order->customer['email_address'], $reason_2);
			$reason_2 = substr($reason_2, 0, 27);
			$user_variable_0 = $order_id;
			$user_variable_1 = $customer_id;
			$user_variable_2 = $_SESSION['cart']->cartID;
			
			$this->sofort->setAmount($total, $currency);
			$this->sofort->setReason(HelperFunctions::convertEncoding($reason_1,3), HelperFunctions::convertEncoding($reason_2,3));
			$this->sofort->addUserVariable(HelperFunctions::convertEncoding($user_variable_0,3));
			$this->sofort->addUserVariable(HelperFunctions::convertEncoding($user_variable_1,3));
			$this->sofort->addUserVariable(HelperFunctions::convertEncoding($user_variable_2,3));
			
			$success_url = tep_href_link(FILENAME_CHECKOUT_PROCESS, '', 'SSL', true, false);
			$success_url = HelperFunctions::cleanUrlParameter($success_url);
			$this->sofort->setSuccessUrl(HelperFunctions::convertEncoding($success_url,3)); 			//will be set in -USER_VARIABLE_3-
			
			$cancel_url = tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error='.$this->code, 'SSL', true, false);
			$cancel_url = HelperFunctions::cleanUrlParameter($cancel_url);
			$this->sofort->setAbortUrl(HelperFunctions::convertEncoding($cancel_url,3));    			//will be set in -USER_VARIABLE_4-
			
			$notification_url = tep_href_link('ext/modules/payment/sofort/callback.php', 'action=ideal', 'SSL', true, false);
			$notification_url = HelperFunctions::cleanUrlParameter($notification_url);
			$this->sofort->setNotificationUrl(HelperFunctions::convertEncoding($notification_url,3));	//will be set in -USER_VARIABLE_5-
			
			$this->sofort->setSenderCountryId('NL');
			$this->sofort->setSenderBankCode($_SESSION['ideal_bank_name']);
			
			$url = $this->sofort->getPaymentUrl();
			$time = date("d.m.Y, G:i:s");
			
			HelperFunctions::insertHistoryEntry((int)$order_id, SOFORT_IDEAL_CLASSIC_TMP_STATUS_ID, SOFORT_IDEAL_CLASSIC_TMP_COMMENT, '', $time);
			
			for ($i=0, $n=sizeof($order->products); $i<$n; $i++) {
				// Stock Update - Joao Correia
				if (STOCK_LIMITED == 'true') {
					if (DOWNLOAD_ENABLED == 'true') {
						$stock_query_raw = "SELECT		products_quantity,
														pad.products_attributes_filename
											FROM		".TABLE_PRODUCTS." p
											LEFT JOIN	".TABLE_PRODUCTS_ATTRIBUTES." pa
											ON			p.products_id=pa.products_id
											LEFT JOIN	".TABLE_PRODUCTS_ATTRIBUTES_DOWNLOAD." pad
											ON			pa.products_attributes_id=pad.products_attributes_id
											WHERE		p.products_id = '".tep_get_prid(HelperFunctions::escapeSql($order->products[$i]['id']))."'
						";
							
						// Will work with only one option for downloadable products
						// otherwise, we have to build the query dynamically with a loop
						$products_attributes = $order->products[$i]['attributes'];
							
						if (is_array($products_attributes)) {
							$stock_query_raw .= " AND pa.options_id = '".HelperFunctions::escapeSql($products_attributes[0]['option_id'])."' AND pa.options_values_id = '".HelperFunctions::escapeSql($products_attributes[0]['value_id'])."'";
						}
							
						$stock_query = tep_db_query($stock_query_raw);
					} else {
						$stock_query = tep_db_query("SELECT products_quantity FROM ".TABLE_PRODUCTS." WHERE products_id = '".tep_get_prid(HelperFunctions::escapeSql($order->products[$i]['id']))."'");
					}
					if (tep_db_num_rows($stock_query) > 0) {
						$stock_values = tep_db_fetch_array($stock_query);
						// do not decrement quantities if products_attributes_filename exists
						$stock_left = (DOWNLOAD_ENABLED != 'true' || !$stock_values['products_attributes_filename']) ? $stock_values['products_quantity'] - $order->products[$i]['qty'] : $stock_values['products_quantity'];
						tep_db_query("UPDATE ".TABLE_PRODUCTS." SET products_quantity = '".HelperFunctions::escapeSql($stock_left)."' WHERE products_id = '".tep_get_prid(HelperFunctions::escapeSql($order->products[$i]['id']))."'");
							
						if ( ($stock_left < 1) && (STOCK_ALLOW_CHECKOUT == 'false') ) {
							tep_db_query("UPDATE ".TABLE_PRODUCTS." SET products_status = '0' WHERE products_id = '".tep_get_prid(HelperFunctions::escapeSql($order->products[$i]['id']))."'");
						}
					}
				}
			}
			
			return $url;
		} else {
			return '';
		}
	}
	
	
	function after_process () {
		// Clear our session data
		// All other session data will be handled in checkout_process.php
		if (isset($_SESSION['cart_pn_sofortueberweisung_ID'])) unset($_SESSION['cart_pn_sofortueberweisung_ID']);
		if (isset($_SESSION['ideal_bank_name'])) unset($_SESSION['ideal_bank_name']);
	}
	
	
	function output_error (){
		return false;
	}
	
	
	function check () {
		if (!isset($this->_check)) {
			$check_query = tep_db_query("SELECT configuration_value FROM ".TABLE_CONFIGURATION." WHERE configuration_key = 'SOFORT_IDEAL_CLASSIC_STATUS'");
			$this->_check = tep_db_num_rows($check_query);
		}
		
		return $this->_check;
	}
	
	
	function get_error () {
		if (isset($_GET['payment_error']) && $_GET['payment_error'] != 'sofort_ideal') {
			return false;
		}
		
		$error = false;
		$errormsg = '';
		
		if (!empty($_GET['payment_error'])) {
			//redirect from SOFORT or abortion by customer or other for customer unimportant error
			$errormsg = SOFORT_IDEAL_CLASSIC_ERROR_DEFAULT;
		}
		
		if(isset($_GET['xmlError'] )) {
			//error in our module/SofortLib
			$errormsg = SOFORT_IDEAL_CLASSIC_ERROR_ALL_CODES;
		}
		
		if(isset($_GET['error_codes'])) { //error is important for customer (e.g. customer-errors)
			$errormsg = '';
			$errorCodes = array_unique (explode(',', HelperFunctions::htmlMask($_GET['error_codes'])));
			
			foreach ($errorCodes as $errorCode) {
				$constant = 'SOFORT_MULTIPAY_XML_FAULT_'.$errorCode;
				
				if (defined ($constant) ) {
					//show specific errorinfo to customer
					$errormsg .= constant ($constant)." ";
				} else {
					//specific errorinfo not defined - should never appear
					$errormsg = SOFORT_IDEAL_CLASSIC_ERROR_ALL_CODES;
					break;
				}
			}
			
			//error exists but no defined errorcode set
			if(empty($errormsg)){
				$errormsg = SOFORT_IDEAL_CLASSIC_ERROR_ALL_CODES ;
			}
		}
		
		return array(
				'title' => SOFORT_IDEAL_CLASSIC_TEXT_ERROR_HEADING,
				'error' => $errormsg);
	}
	
	
	/**
	 * Check, if needed sofort-lang-constants exists.
	 * If not, include the english-lang-file(s).
	 * @return always true
	 */
	function _checkExistingSofortConstants() {
		//security check - constant exists if lang-file exists
		if(defined('SOFORT_IDEAL_TEXT_TITLE')) {
			return true;
		}
		
		$lngdir = DIR_FS_CATALOG.'includes/languages/';
		$lng = $_SESSION['language'];
		
		$iterator = new DirectoryIterator($lngdir);
		
		foreach ($iterator as $file){
			if ($file->isDir() && file_exists($lngdir.$file->getFilename().'/modules/payment/sofort_general.php')) $installedModulLangs[] = $file->getFilename();
		}
		
		//currently installed in this module
		if(in_array($lng, $installedModulLangs)) {
			include_once($lngdir.strtolower($lng).'/modules/payment/sofort_ideal.php');
		} else {
			include_once($lngdir.'english/modules/payment/sofort_ideal.php');
		}
		
		return true;
	}
	
	
	/**
	 * Insert all SOFORT-Status into table orders_status and return them
	 * @return array with all status
	 */
	function _insertAndReturnSofortStatus() {
		require_once(DIR_FS_CATALOG.'ext/modules/payment/sofort/sofortInstallIdeal.php');
		
		//SOFORT-langs in this module
		$lngdir = DIR_FS_CATALOG.'includes/languages/';
		$iterator = new DirectoryIterator($lngdir);
		
		foreach ($iterator as $file){
			if ($file->isDir() && file_exists($lngdir.$file->getFilename().'/modules/payment/sofort_general.php')) $sofortLangs[] = strtoupper($file->getFilename());
		}
		
		//current installed langs
		$installedLangs = array();
		$orderQuery = tep_db_query("SELECT languages_id, directory FROM ".HelperFunctions::escapeSql(TABLE_LANGUAGES));
		
		while ($result = tep_db_fetch_array($orderQuery)) {
			$installedLangs[$result['languages_id']] = strtoupper($result['directory']);
		}
		
		//get the current highest orders_status_id
		$orderQuery = tep_db_query("SELECT MAX(orders_status_id) AS max_orders_status_id FROM orders_status");
		$maxOrdersStatusIdTemp = tep_db_fetch_array($orderQuery);
		$ordersStatusIdTemp = $maxOrdersStatusIdTemp['max_orders_status_id'] + 1;
		$ordersStatusIdConfirmed = $ordersStatusIdTemp + 1;
		$ordersStatusIdAborted = $ordersStatusIdConfirmed + 1;
		$ordersStatusIdLoss = $ordersStatusIdAborted + 1;
		$ordersStatusIdRefCom = $ordersStatusIdLoss + 1;
		$ordersStatusIdRefRef = $ordersStatusIdRefCom + 1;
		$sofortStatuses = array();
		
		foreach($installedLangs as $installedLang) {
			//insert english for shop-languages, which are not included in this module
			if(in_array($installedLang, $sofortLangs)) {
				$sofortLang = $installedLang;
				$langId = array_search($sofortLang, $installedLangs);
			} else {
				$sofortLang = 'ENGLISH';
				$langId = array_search($installedLang, $installedLangs);
			}
			
			//insert temp-Status if not exists
			$newOrdersStatusName = $this->_getNewOrdersStatusName('ideal_temp', $sofortLang);
			$sofortStatuses['ideal_temp'] = $this->_getStatusIdIfExistInDb($newOrdersStatusName, $langId);
			
			if($sofortStatuses['ideal_temp'] === false || $sofortStatuses['ideal_temp'] == '') {
				$sofortStatuses['ideal_temp'] = $this->_insertStatusInDb($newOrdersStatusName, $ordersStatusIdTemp, $langId);
			}
			
			//insert confirmed-Status if not exists
			$newOrdersStatusName = $this->_getNewOrdersStatusName('ideal_confirmed', $sofortLang);
			$sofortStatuses['ideal_confirmed'] = $this->_getStatusIdIfExistInDb($newOrdersStatusName, $langId);
			
			if($sofortStatuses['ideal_confirmed'] === false || $sofortStatuses['confirmed'] == '') {
				$sofortStatuses['ideal_confirmed'] = $this->_insertStatusInDb($newOrdersStatusName, $ordersStatusIdConfirmed, $langId);
			}
			
			//insert aborted-Status if not exists
			$newOrdersStatusName = $this->_getNewOrdersStatusName('ideal_aborted', $sofortLang);
			$sofortStatuses['ideal_aborted'] = $this->_getStatusIdIfExistInDb($newOrdersStatusName, $langId);
			
			if($sofortStatuses['ideal_aborted'] === false || $sofortStatuses['ideal_aborted'] == '') {
				$sofortStatuses['ideal_aborted'] = $this->_insertStatusInDb($newOrdersStatusName, $ordersStatusIdAborted, $langId);
			}
			
			//insert loss-Status if not exists
			$newOrdersStatusName = $this->_getNewOrdersStatusName('ideal_loss', $sofortLang);
			$sofortStatuses['ideal_loss'] = $this->_getStatusIdIfExistInDb($newOrdersStatusName, $langId);
			
			if($sofortStatuses['ideal_loss'] === false || $sofortStatuses['ideal_loss'] == '') {
				$sofortStatuses['ideal_loss'] = $this->_insertStatusInDb($newOrdersStatusName, $ordersStatusIdLoss, $langId);
			}
			
			//insert refunded_compensation-Status if not exists
			$newOrdersStatusName = $this->_getNewOrdersStatusName('ideal_ref_com', $sofortLang);
			$sofortStatuses['ideal_ref_com'] = $this->_getStatusIdIfExistInDb($newOrdersStatusName, $langId);
			
			if($sofortStatuses['ideal_ref_com'] === false || $sofortStatuses['ideal_ref_com'] == '') {
				$sofortStatuses['ideal_ref_com'] = $this->_insertStatusInDb($newOrdersStatusName, $ordersStatusIdRefCom, $langId);
			}
			
			//insert refunded_refunded-Status if not exists
			$newOrdersStatusName = $this->_getNewOrdersStatusName('ideal_ref_ref', $sofortLang);
			$sofortStatuses['ideal_ref_ref'] = $this->_getStatusIdIfExistInDb($newOrdersStatusName, $langId);
			
			if($sofortStatuses['ideal_ref_ref'] === false || $sofortStatuses['ideal_ref_ref'] == '') {
				$sofortStatuses['ideal_ref_ref'] = $this->_insertStatusInDb($newOrdersStatusName, $ordersStatusIdRefRef, $langId);
			}
		}
		
		return  $sofortStatuses;
	}
	
	
	function _getStatusIdIfExistInDb($newOrdersStatusName, $langId) {
		if(!$newOrdersStatusName) return false;
		
		$checkSql = 'SELECT	orders_status_id
					 FROM	orders_status
					 WHERE	language_id = "'.HelperFunctions::escapeSql($langId).'"
					 AND	orders_status_name = "'.HelperFunctions::escapeSql($newOrdersStatusName).'"
					 LIMIT	1';
		$checkQuery = tep_db_query($checkSql);
		
		if (tep_db_num_rows($checkQuery) < 1) {
			return false;
		} else {
			$neededOrdersStatusId = tep_db_fetch_array($checkQuery);
			return $neededOrdersStatusId['orders_status_id'];
		}
	}
	
	
	/**
	 * insert given statusstring into DB - empty strings will not be inserted
	 * @return $orders_status_id from DB OR false
	 */
	function _insertStatusInDb($newOrdersStatusName, $ordersStatusId, $langId) {
		if (!$newOrdersStatusName) return false;
		
		tep_db_query("INSERT INTO orders_status (orders_status_id, language_id, orders_status_name) VALUES ('".(int)$ordersStatusId."', '".(int)$langId."', '".HelperFunctions::escapeSql($newOrdersStatusName)."')");
		return $ordersStatusId;
	}
	
	
	/**
	 * returns the statusname for the given $status and $lang
	 * return max the first 32 chars! (because db-field = varchar(32) )
	 */
	function _getNewOrdersStatusName($status, $lang) {
		switch ($status) {
			case 'ideal_temp': 		$newOrdersStatusName = constant('SOFORT_IDEAL_STATE_TEMP_'.$lang); break;
			case 'ideal_confirmed':	$newOrdersStatusName = constant('SOFORT_IDEAL_STATE_CONFIRMED_'.$lang); break;
			case 'ideal_loss':		$newOrdersStatusName = constant('SOFORT_IDEAL_STATE_LOSS_'.$lang); break;
			case 'ideal_aborted':	$newOrdersStatusName = constant('SOFORT_IDEAL_STATE_ABORTED_'.$lang); break;
			case 'ideal_ref_com':	$newOrdersStatusName = constant('SOFORT_IDEAL_STATE_REF_COM_'.$lang); break;
			case 'ideal_ref_ref':	$newOrdersStatusName = constant('SOFORT_IDEAL_STATE_REF_REF_'.$lang); break;
		}
		// if string is not cut to 32 chars, status will be reinserted with every installation
		$newOrdersStatusName = substr($newOrdersStatusName, 0, 32);
		
		return $newOrdersStatusName;
	}
	
	
	/**
	 * check if order has temp-Status and buyer didnt finish the payment-process by "normal" way
	 * @return bool true (always)
	 */
	function _checkCancelOrder() {
		global $cart_pn_ID, $order;
		
		//tmp-order exists, if following if is true
		if (tep_session_is_registered('cart_pn_ID')) {
			$orderId = substr($cart_pn_ID, strpos($cart_pn_ID, '-')+1);
			$pnCartID = substr($cart_pn_ID, 0, strlen($_SESSION['cart']->cartID));
			$tempStatus = SOFORT_IDEAL_CLASSIC_TMP_STATUS_ID;
			
			$checkQuery = tep_db_query("SELECT orders_status FROM ".HelperFunctions::escapeSql(TABLE_ORDERS)." WHERE orders_id = '".(int)$orderId."' LIMIT 1");
			$result = tep_db_fetch_array($checkQuery);
				
			if ($result['orders_status'] == $tempStatus && $_SESSION['cart']->cartID == $pnCartID) {
				tep_session_unregister('cart_pn_ID');
				$this->_cancelOrder((int)$orderId, 'on');
			}
		}
		
		return true;
	}
	
	
	/**
	 * @param bool $restock - only allowed is: FALSE or 'on'
	 */
	function _cancelOrder($orderId, $restock) {
		if ($restock == 'on') {
			$order_query = tep_db_query("SELECT orders_products_id, products_id, products_quantity
										 FROM	".TABLE_ORDERS_PRODUCTS."
										 WHERE	orders_id = '".(int)$orderId."'");
			
			while ($order = tep_db_fetch_array($order_query)) {
				tep_db_query("UPDATE ".TABLE_PRODUCTS."
							  SET	 products_quantity = products_quantity + ".(int)$order['products_quantity'].", products_ordered = products_ordered - ".(int)$order['products_quantity']."
							  WHERE	 products_id = '".HelperFunctions::escapeSql($order['products_id'])."'");
			}
		}
		
		//update status and customer-history
		$time = date("d.m.Y, G:i:s");
		tep_db_query('UPDATE '.TABLE_ORDERS.' SET orders_status = "'.(int)SOFORT_IDEAL_CLASSIC_CANCELED_ORDER_STATUS_ID.'", last_modified = now() WHERE orders_id = '.(int)$orderId);
		HelperFunctions::insertHistoryEntry((int) $orderId, SOFORT_IDEAL_CLASSIC_CANCELED_ORDER_STATUS_ID, SOFORT_IDEAL_CLASSIC_ORDER_CANCELED, '', $time);
	}
	
	
	function cleanString($string) {
		return htmlentities(strip_tags($string), ENT_QUOTES, HelperFunctions::getIniValue('shopEncoding'));
	}
}
?>