<?php
/**
 * @version SOFORT Gateway 3.3.0 - $Date: 2013-05-23 15:34:28 +0200 (Thu, 23 May 2013) $
 * @author SOFORT AG (integration@sofort.com)
 * @link http://www.sofort.com/
 *
 * Copyright (c) 2012 SOFORT AG
 *
 * Released under the GNU General Public License (Version 2)
 * [http://www.gnu.org/licenses/gpl-2.0.html]
 *
 * $Id: sofort_sr.php 6160 2013-05-23 13:34:28Z rotsch $
 */

require_once(DIR_FS_CATALOG.'ext/modules/payment/sofort/sofort.php');
require_once(DIR_FS_CATALOG.'ext/modules/payment/sofort/library/sofortLib.php');

class sofort_sr extends sofort {
	
	function sofort_sr() {
		global $order;
		
		parent::sofort();
		
		$this->_checkExistingSofortConstants('sr');
		
		//if(isset($_SESSION['sofort']['sofort_conditions_sr'])) unset($_SESSION['sofort']['sofort_conditions_sr']);
		
		$this->code = 'sofort_sr';
		$this->code_short = 'sr';
		$this->paymentMethod = 'SR';
		$this->title = SOFORT_SR_TEXT_TITLE;
		$this->public_title = SOFORT_SR_TEXT_PUBLIC_TITLE;

		if (defined('SOFORT_SR_RECOMMENDED_PAYMENT') && SOFORT_SR_RECOMMENDED_PAYMENT == 'True') $this->public_title .= ' ' . SOFORT_SR_RECOMMENDED_PAYMENT_TEXT;
		
		$this->enabled = ((defined('SOFORT_SR_STATUS') && SOFORT_SR_STATUS == 'True') ? true : false);
		$this->description = SOFORT_SR_TEXT_DESCRIPTION.'<br />'.SOFORT_MULTIPAY_VERSIONNUMBER.': '.HelperFunctions::getSofortmodulVersion();
		
		if ($this->_isInstalled() && (!defined('SOFORT_MULTIPAY_MODULE_VERSION') || strcasecmp(trim(SOFORT_MULTIPAY_MODULE_VERSION), trim(HelperFunctions::getSofortmodulVersion())) != 0)) {
			$this->description = '<span style ="color:red; font-weight: bold; font-size: 1.2em">'.SOFORT_MULTIPAY_UPDATE_NOTICE.'</span><br /><br />'.$this->description;
		}
		
		$this->description .= SOFORT_SR_TEXT_DESCRIPTION_EXTRA;
		$this->sort_order = (defined('SOFORT_SR_SORT_ORDER') ? SOFORT_SR_SORT_ORDER : false);
		
		if (is_object($order)) $this->update_status();
		
		if (defined('SOFORT_SR_STATUS')) {
			$this->invoice = new PnagInvoice(SOFORT_MULTIPAY_CONFIGURATION_KEY, '');
			$this->invoice->setVersion(HelperFunctions::getSofortmodulVersion());
			
			if (defined('SOFORT_MULTIPAY_LOG_ENABLED') && SOFORT_MULTIPAY_LOG_ENABLED == "True") $this->invoice->enableLog();
		}
	}
	
	
	function selection () {
		if (!parent::selection()) {
			$this->invoice->log("Notice: Paymentmethod ".$this->code." will be deactivated for selection.");
			$this->enabled = false;
			
			return false;
		}
		
		//virtual content with SR is not allowed
		if ($this->_orderHasVirtualProducts() || $this->_deliveryAddressDoesNotExist()) {
			$this->invoice->log("Notice: Paymentmethod ".$this->code." will be deactivated for selection because of found virtual products in cart.");
			$this->enabled = false;
			
			return false;
		}
		
		$fields = array(
				array(
						'title' => SOFORT_SR_CHECKOUT_CONDITIONS,
						'field' => tep_draw_checkbox_field('sofort_conditions_sr', 'sofort_conditions_sr')
				)
		);
		
		return array(
				'id' => $this->code,
				'module' => $this->public_title,
				'fields' => $fields
		);
	}
	
	
	function before_process() {
		parent::before_process();
		
		//downloads and gift-voucher are not allowed with SR
		if ($this->_orderHasVirtualProducts() || $this->_deliveryAddressDoesNotExist()) {
			$this->invoice->log("Notice: ".$this->code." selected for payment. Redirect to cancelUrl because of found virtual products in cart. Time: ".date("d.m.Y, G:i:s"));
			$errors = array(0 => array('code' => '10003'));
			tep_redirect(HelperFunctions::getCancelUrl($this->code, $errors));
		}
	}
	
	
	function pre_confirmation_check() {
		parent::pre_confirmation_check();
		
		$requestData = $_POST;
		$requestData['sofort_conditions_sr'] = trim($requestData['sofort_conditions_sr']);
		$_SESSION['sofort']['sofort_conditions_sr'] = isset($requestData['sofort_conditions_sr']) ? $requestData['sofort_conditions_sr'] : '';
		
		if ($_SESSION['sofort']['sofort_conditions_sr'] != 'sofort_conditions_sr') {
			$payment_error_return = 'payment_error='.$this->code.'&error_codes=10002';
			$redirectUrl = tep_href_link(FILENAME_CHECKOUT_PAYMENT, $payment_error_return, 'SSL', true, false);
			tep_redirect(HelperFunctions::cleanUrlParameter($redirectUrl));
		}
		
		return false;
	}
	
	
	function process_button() {
		global $order, $customer_id, $cart_pn_ID, $currency, $currencies, $insert_id, $order_total, $redirect;
		
		if ($redirect === true) {
			$orderId = substr($cart_pn_ID, strpos($cart_pn_ID, '-')+1);
			$reasons = $this->_getReasons($this->paymentMethod, $customer_id, $order, $orderId);
			$paymentSecret = md5(mt_rand().microtime());
			$userVariable_0 = $orderId;
			$userVariable_1 = $customer_id;
			$userVariable_2 = $_SESSION['cart']->cartID;
			$userVariable_3 = $paymentSecret;
			
			$successUrl = tep_href_link(FILENAME_CHECKOUT_PROCESS, '', 'SSL', true, false);
			$successUrl = HelperFunctions::cleanUrlParameter($successUrl);
			
			$cancelUrl = tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error='.$this->code.'&tid=-TRANSACTION-', 'SSL', true, false);
			$cancelUrl = HelperFunctions::cleanUrlParameter($cancelUrl);
			
			$notificationUrl = tep_href_link('ext/modules/payment/sofort/callback.php', 'paymentSecret='.$paymentSecret.'&action=multipay', 'SSL', true, false);
			$notificationUrl = HelperFunctions::cleanUrlParameter($notificationUrl);
			
			$this->invoice->setCurrency($currency);  //others than EUR will currently not be accepted by API!
			$this->invoice->setReason(HelperFunctions::convertEncoding($reasons[0],3), HelperFunctions::convertEncoding($reasons[1],3));
			$this->invoice->setSuccessUrl(HelperFunctions::convertEncoding($successUrl,3));
			$this->invoice->setAbortUrl(HelperFunctions::convertEncoding($cancelUrl,3));
			$this->invoice->setTimeoutUrl(HelperFunctions::convertEncoding($cancelUrl,3));
			$this->invoice->setNotificationUrl(HelperFunctions::convertEncoding($notificationUrl,3));
			$this->invoice->addUserVariable(HelperFunctions::convertEncoding($userVariable_0,3));
			$this->invoice->addUserVariable(HelperFunctions::convertEncoding($userVariable_1,3));
			$this->invoice->addUserVariable(HelperFunctions::convertEncoding($userVariable_2,3));
			$this->invoice->addUserVariable(HelperFunctions::convertEncoding($userVariable_3,3));
			$this->invoice->setEmailCustomer(HelperFunctions::convertEncoding($order->customer['email_address'],3));
			$this->invoice->setPhoneNumberCustomer($order->customer['telephone']);
			$this->invoice->setSofortrechnung();
			$this->invoice->setCustomerId(HelperFunctions::convertEncoding($customer_id,3));
			$this->invoice->setOrderId($orderId);
			$this->invoice->setDebitorVatNumber(HelperFunctions::convertEncoding($_SESSION['customer_vat_id'], 3));
			
			$this->_addCustomerAddressesToInvoice();
			$this->_addProductsToInvoice();
			$shippingItemId = $this->_addShippingToInvoice();
			$this->_addPriceModificatorsToInvoice($order_totals);
			
			//check shopTotal against the invoiceclassTotal
			//if the difference is bigger than 1%: articles/discounts/etc. have not been given correct to $this->invoice
			$shopTotal = number_format($order->info['total'] * $currencies->get_value($currency), 2, '.','');
			$invoiceTotal = $this->invoice->getAmount();
			
			if (!$this->_checkShopTotalVsInvoiceTotal($shopTotal, $invoiceTotal) ) {
				$this->invoice->logError($this->code.": ShopTotal ($shopTotal) is not near InvoiceTotal ($invoiceTotal)! Customer-ID ".$customer_id.".
									Are you using price-modification-tools which are not supported by this 'Rechnung bei SOFORT'-module? Was the tax-rate
									of all affected articles, shipping options and other positions set correctly? Did you set the currency-exchange-rate
									correctly? Buyer redirected to cancel-URL.");
				$errors = array(0 => array('code' => '10007'));
				tep_redirect(HelperFunctions::getCancelUrl($this->code, $errors));
			}
			
			//send all data to the API and place an order at SOFORT if no errors
			$this->invoice->checkout();
			
			if($this->invoice->isError()) {
				$this->invoice->logWarning("API-Call returned false. Redirect to cancel-URL. API-Errors: ".print_r($this->invoice->getErrors(), true)." Time: ".date("d.m.Y, G:i:s"));
				tep_redirect(HelperFunctions::getCancelUrl($this->code, $this->invoice->getErrors()));
			} else {
				$url = $this->invoice->getPaymentUrl();
				$transactionId = $this->invoice->getTransactionId();
				
				//seller and customer comment
				$time = date("d.m.Y, G:i:s");
				
				//set temp-status (only table orders_history, not table orders)
				$tmpStatusId = SOFORT_MULTIPAY_TEMP_STATUS_ID;
				
				//dont set a new status? -> then use the last order status
				if (HelperFunctions::statusIsUnchangedStatus($tmpStatusId)) {
					$tmpStatusId = HelperFunctions::getLastOrderStatus($orderId);
				}
				
				$tmpStatusId = HelperFunctions::checkStatusId($tmpStatusId);
				$tmpComment = SOFORT_SR_TMP_COMMENT.' '.SOFORT_TRANSLATE_TIME.': '.$time;
				HelperFunctions::insertHistoryEntry((int)$orderId, $tmpStatusId, $tmpComment);
				
				//comment only for seller
				$tmpCommentSeller = constant('SOFORT_SR_TMP_COMMENT_SELLER').' '.SOFORT_TRANSLATE_TIME.': '.$time;
				HelperFunctions::insertHistoryEntry((int)$orderId, -1, $tmpCommentSeller, 0);
				
				//save transaction data
				HelperFunctions::insertHistoryEntry((int)$orderId, -1, $transactionId, 0);
				
				//save all important data in our sofort-tables
				$sofortOrderId = HelperFunctions::insertSofortOrder($orderId, $paymentSecret, $transactionId, $this->paymentMethod);
				
				if (!$sofortOrderId) {
					$this->invoice->logWarning("Warning: Saving of orderdetails in table sofort_orders failed. Function insertSofortOrder() returned false.
						Given params: OrderId: $orderId, PaymentSecret: $paymentSecret, TransId: $transactionId, PaymentMethod:".$this->paymentMethod);
				}
				
				if($shippingItemId) {
					$sofortShippingSaved = $this->insertShippingCostsInSofortTables($orderId, $shippingItemId);
					
					if(!$sofortShippingSaved){
						$this->invoice->logWarning("Warning: Saving of shipping_id in table sofort_products failed. Function insertShippingCostsInSofortTables() returned false.
								Given params: OrderId: $orderId - shippingItemId: $shippingItemId");
					}
				}
				
				$sofortProductsSaved = $this->insertOrderAttributesInSofortTables($orderId);
				
				if(!$sofortProductsSaved){
					$this->invoice->logWarning("Warning: Saving of orders_products_id in table sofort_products failed. Function insertOrderAttributesInSofortTables() returned false. Given params: OrderId: $orderId");
				}
				
				$_SESSION['sofort']['sofort_payment_url'] = $url;
				$_SESSION['sofort']['sofort_payment_method'] = $this->code;
				
				//following file will always redirect to SOFORT-Wizard (or to Error-URL)
				return tep_href_link('ext/modules/payment/sofort/ressources/scripts/processSofortPayment.php', '', 'SSL', true, false);
			}
		} else {
			return '';
		}
	}
	
	
	function _addCustomerAddressesToInvoice() {
		global $order;
		
		//split address into street and number at last dot or space
		if(!preg_match('#(.+)[ .](.+)#i', trim($order->billing['street_address']), $billing)) {
			$billing = array();
			$billing[1] = trim($order->billing['street_address']);
			$billing[2] = '';
		}
		
		if(!preg_match('#(.+)[ .](.+)#i', trim($order->delivery['street_address']), $delivery)) {
			$delivery = array();
			$delivery[1] = trim($order->delivery['street_address']);
			$delivery[2] = '';
		}
		
		$billingCompanyName = trim($order->billing['company']);
		$billingNameAdditive = '';
		
		//get billing-salutation: 2=masculine, 3=feminine
		$billingSalutation = $this->_getGenderFromAddressBook($order->billing['firstname'], $order->billing['lastname'], $order->billing['company'], $order->billing['street_address'], $order->billing['postcode'], $order->billing['city'],
															  $order->billing['country_id'], $order->billing['zone_id']);
		$deliveryCompanyName = trim($order->delivery['company']);
		$deliveryNameAdditive = '';
		
		//get deliver-salutation: 2=masculine, 3=feminine
		$deliverSalutation = $this->_getGenderFromAddressBook($order->delivery['firstname'], $order->delivery['lastname'], $order->delivery['company'], $order->delivery['street_address'], $order->delivery['postcode'], $order->delivery['city'],
															  $order->delivery['country_id'], $order->delivery['zone_id']);
		
		$this->invoice->addInvoiceAddress(HelperFunctions::convertEncoding($order->billing['firstname'],3), HelperFunctions::convertEncoding($order->billing['lastname'],3), HelperFunctions::convertEncoding($billing[1],3),
										  HelperFunctions::convertEncoding($billing[2],3), $order->billing['postcode'], HelperFunctions::convertEncoding($order->billing['city'],3), $billingSalutation, $order->billing['country']['iso_code_2'],
										  HelperFunctions::convertEncoding($billingNameAdditive, 3), HelperFunctions::convertEncoding($order->billing['suburb'],3), HelperFunctions::convertEncoding($billingCompanyName,3));
		$this->invoice->addShippingAddress(HelperFunctions::convertEncoding($order->delivery['firstname'],3), HelperFunctions::convertEncoding($order->delivery['lastname'],3), HelperFunctions::convertEncoding($delivery[1],3),
										   HelperFunctions::convertEncoding($delivery[2],3), $order->delivery['postcode'], HelperFunctions::convertEncoding($order->delivery['city'],3), $deliverSalutation, $order->delivery['country']['iso_code_2'],
										   HelperFunctions::convertEncoding($deliveryNameAdditive, 3), HelperFunctions::convertEncoding($order->delivery['suburb'],3), HelperFunctions::convertEncoding($deliveryCompanyName,3));
		
		return true;
	}
	
	
	/**
	 * add all bought products to $this->invoice
	 */
	function _addProductsToInvoice() {
		global $order, $currencies, $currency;
		
		foreach($order->products as $product) {
			//get attributes and add as description to item
			$description = '';
			
			if ((isset ($product['attributes'])) && (sizeof($product['attributes']) > 0)) {
				foreach ($product['attributes'] as $attribute) $description .= $attribute['option'] . ": " . $attribute['value'] . "\n";
				
				$description = trim($description); //remove last \n
			}
			
			$description = helperFunctions::convertEncoding(html_entity_decode($description, ENT_QUOTES, HelperFunctions::getIniValue('shopEncoding')),3);
			$amount = $product['final_price'] * $currencies->get_value($currency);
			$amount = $this->_addTax ($amount, $product['tax']);
			$amount = number_format ($amount, 2, '.','');
			$itemId = HelperFunctions::sortSofortItemId($product['id']);
			$this->invoice->addItemToInvoice(helperFunctions::convertEncoding($itemId,3), helperFunctions::convertEncoding($product['model'],3), helperFunctions::convertEncoding($product['name'],3), $amount, 0, $description, $product['qty'], $product['tax']);
		}
	}
	
	
	/**
	 * add shippinginfo to $this->invoice
	 */
	function _addShippingToInvoice() {
		global $order, $currencies, $currency, $shipping_modules;
		
		//add shipping to cart
		$shipping = $shipping_modules->quote();
		
		if($shipping){
			list ($shippingModule, $shippingMethod) = explode('_', $_SESSION['shipping']['id']);
			$itemId =  'shipping|'.substr($shippingModule.'|'.$shippingMethod,0,22);
			
			if (isset ($shipping[0]['tax']) ) {
				$shippingCosts = $order->info['shipping_cost'] * $currencies->get_value($currency);
				
				if (defined('DISPLAY_PRICE_WITH_TAX') && DISPLAY_PRICE_WITH_TAX == 'false') $shippingCosts = $this->_addTax($shippingCosts, $shipping[0]['tax']);
				
				$shippingCosts = number_format($shippingCosts, 2, '.','');
				$shippingTitle = HelperFunctions::convertEncoding(html_entity_decode($order->info['shipping_method'], ENT_QUOTES, HelperFunctions::getIniValue('shopEncoding')),3);
				$this->invoice->addItemToInvoice($itemId, '', $shippingTitle, $shippingCosts, 1, '', 1, $shipping[0]['tax']);
			} else {
				$shippingCosts = $order->info['shipping_cost'] * $currencies->get_value($currency);
				
				//activate and edit following lines if $shipping doesnt exist AND DISPLAY_PRICE_WITH_TAX == false
				//@see: Shopbackend -> Configuration -> My Store -> Display Prices with tax
				//$taxValue = ???; //Notice: $order->info['shipping_cost'] may not have the correct tax-value included. Check before!
				//$shippingCosts = tep_add_tax($shippingCosts, $taxValue);
				$shippingCosts = number_format($shippingCosts, 2, '.','');
				$shippingTitle = HelperFunctions::convertEncoding(html_entity_decode($order->info['shipping_method'], ENT_QUOTES, HelperFunctions::getIniValue('shopEncoding')),3);
				$this->invoice->addItemToInvoice($itemId, '', $shippingTitle, $shippingCosts, 1);
			}
			
			return $itemId;
		}
		
		return false;
	}
	
	
	/**
	 * add discounts or agio (e.g. ot_sofort, loworderfee, discount...) to $this->invoice
	 */
	function _addPriceModificatorsToInvoice($orderTotals) {
		global $currencies, $currency;
		
		//check optional price-modificators like loworderfee
		if(is_array($orderTotals) ) {
			foreach($orderTotals as $totalModule) {
				$itemId =  'discount|'.substr($totalModule['code'],0,22);
				
				if($totalModule['code'] == 'ot_loworderfee') { //checked ok
					//$md5String = 'ot_loworderfee|'.$order_id.mt_rand();
					$amount = $totalModule['value'] * $currencies->get_value($currency);
					$taxRate = tep_get_tax_rate(MODULE_ORDER_TOTAL_LOWORDERFEE_TAX_CLASS);
					
					if (defined('DISPLAY_PRICE_WITH_TAX') && DISPLAY_PRICE_WITH_TAX == 'false') $amount = $this->_addTax($amount, $taxRate);
					
					$lowOrderFeeTitle = helperFunctions::convertEncoding(html_entity_decode($totalModule['title'], ENT_QUOTES, HelperFunctions::getIniValue('shopEncoding')),3);
					$this->invoice->addItemToInvoice($itemId, '', $lowOrderFeeTitle, $amount, 2, '', 1, $taxRate);
				}
			}
		}
		
		return true;
	}
	
	
	function confirmation ($confirmationMessage = '') {
		return parent::confirmation(SOFORT_SR_CHECKOUT_CONFIRMATION);
	}
	
	
	function install() {
		$sofortStatuses = $this->_insertAndReturnSofortStatus();
		$canceledStatus = (isset($sofortStatuses['canceled']) && !empty($sofortStatuses['canceled'])) ? $sofortStatuses['canceled'] : '';
		$confirmedStatus = (isset($sofortStatuses['invoice_confirmed']) && !empty($sofortStatuses['invoice_confirmed'])) ? $sofortStatuses['invoice_confirmed'] : '';
		$unconfirmedStatus =(isset($sofortStatuses['unconfirmed']) && !empty($sofortStatuses['unconfirmed'])) ? $sofortStatuses['unconfirmed'] : '';
		
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value,  configuration_group_id, sort_order, set_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_SR_STATUS_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_SR_STATUS_DESC)."', 'SOFORT_SR_STATUS', 'False', '6', '1', 'tep_cfg_select_option(array(\'True\', \'False\'), ', now())");
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value,  configuration_group_id, sort_order, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_SR_SORT_ORDER_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_SR_SORT_ORDER_DESC)."', 'SOFORT_SR_SORT_ORDER', '0', '6', '16', now())");
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value,  configuration_group_id, sort_order, set_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_SR_RECOMMENDED_PAYMENT_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_SR_RECOMMENDED_PAYMENT_DESC)."', 'SOFORT_SR_RECOMMENDED_PAYMENT', 'False', '6', '5', 'tep_cfg_select_option(array(\'True\', \'False\'), ', now())");
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value,  configuration_group_id, sort_order, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_SOFORTRECHNUNG_ALLOWED_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_SOFORTRECHNUNG_ALLOWED_DESC)."', 'SOFORT_SOFORTRECHNUNG_ALLOWED', '', '6', '12', now())");
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value,  configuration_group_id, sort_order, use_function, set_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_SR_ZONE_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_SR_ZONE_DESC)."', 'SOFORT_SR_ZONE', '0', '6', '13', 'tep_get_zone_class_title', 'tep_cfg_pull_down_zone_classes(', now())");
		
		//"Tempor�r - Rechnung unbest�tigt" - pending-confirm_invoice
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value,  configuration_group_id, sort_order, set_function, use_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_SR_PEN_CON_INV_STATUS_ID_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_SR_PEN_CON_INV_STATUS_ID_DESC)."', 'SOFORT_SR_PEN_CON_INV_STATUS_ID', '".HelperFunctions::escapeSql($unconfirmedStatus)."',  '6', '35', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
		
		//"Best�tigt - Rechnung wurde best�tigt" - pending-not_credited_yet
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value,  configuration_group_id, sort_order, set_function, use_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_SR_PEN_NOT_CRE_YET_STATUS_ID_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_SR_PEN_NOT_CRE_YET_STATUS_ID_DESC)."', 'SOFORT_SR_PEN_NOT_CRE_YET_STATUS_ID', '".HelperFunctions::escapeSql($confirmedStatus)."',  '6', '35', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
		
		//"Stornierung (VOR Best�tigung)" - loss-canceled
		//"Automatische Stornierung (Best�tigungszeitraum abgelaufen)" - loss-confirmation_period_expired
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value,  configuration_group_id, sort_order, set_function, use_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_SR_LOS_CAN_STATUS_ID_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_SR_LOS_CAN_STATUS_ID_DESC)."', 'SOFORT_SR_LOS_CAN_STATUS_ID', '".HelperFunctions::escapeSql($canceledStatus)."',  '6', '35', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
		
		//"Stornierung NACH Best�tigung (=Gutschrift)" - refunded-refunded
		tep_db_query("INSERT INTO ".TABLE_CONFIGURATION." (configuration_title, configuration_description, configuration_key, configuration_value,  configuration_group_id, sort_order, set_function, use_function, date_added) VALUES ('".HelperFunctions::escapeSql(SOFORT_SR_REF_REF_STATUS_ID_TITLE)."', '".HelperFunctions::escapeSql(SOFORT_SR_REF_REF_STATUS_ID_DESC)."', 'SOFORT_SR_REF_REF_STATUS_ID', '".HelperFunctions::escapeSql($canceledStatus)."',  '6', '35', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
		
		//install shared keys, that are used by all/most multipay-modules
		parent::install();
	}
	
	
	function remove() {
		tep_db_query("DELETE FROM ".TABLE_CONFIGURATION." WHERE configuration_key LIKE 'SOFORT_SR%'");
		tep_db_query("DELETE FROM ".TABLE_CONFIGURATION." WHERE configuration_key LIKE 'SOFORT_SOFORTRECHNUNG%'");
		
		//if this is the last removing of a multipay-paymentmethod --> we also remove all shared keys, that are used by all/most multipay-modules
		parent::remove();
	}
	
	
	function keys() {
		
		parent::keys();
		
		return array(
			'SOFORT_SR_STATUS',
			'SOFORT_MULTIPAY_CONFIGURATION_KEY',
			'SOFORT_MULTIPAY_AUTH',
			'SOFORT_SR_RECOMMENDED_PAYMENT',
			'SOFORT_SOFORTRECHNUNG_ALLOWED',
			'SOFORT_SR_ZONE',
			'SOFORT_SR_SORT_ORDER',
			'SOFORT_MULTIPAY_PROF_SETTINGS',
			'SOFORT_MULTIPAY_TEMP_STATUS_ID',
			'SOFORT_MULTIPAY_ABORTED_STATUS_ID',
			'SOFORT_SR_PEN_CON_INV_STATUS_ID',
			'SOFORT_SR_PEN_NOT_CRE_YET_STATUS_ID',
			'SOFORT_SR_LOS_CAN_STATUS_ID',
			'SOFORT_SR_REF_REF_STATUS_ID',
			//'SOFORT_MULTIPAY_LOG_ENABLED',
		);
	}
	
	
	/**
	 * if there is more than 1% difference this function returns false
	 * @return bool
	 */
	function _checkShopTotalVsInvoiceTotal ($shopTotal, $invoiceTotal) {
		$percent = ($shopTotal < $invoiceTotal) ? $shopTotal/$invoiceTotal : $invoiceTotal/$shopTotal;
		
		return ($percent < 0.99) ? false : true;
	}
	
	
	/**
	 * search for downloads and gift-voucher in cart
	 * @return bool
	 */
	function _orderHasVirtualProducts() {
		global $order;
		
		if (!is_object($order)) return false;
		if ($order->content_type == 'virtual') return true;
		
		//search for downloads
		if ($_SESSION['cart']->count_contents() > 0) {
			$cartContents = $_SESSION['cart']->contents;
			reset($cartContents);
			
			foreach($cartContents as $key => $value) {
				if (isset($cartContents[$key]['attributes'])) {
					$productId = explode('{', $key);
					reset($cartContents[$key]['attributes']);
					
					foreach ($cartContents[$key]['attributes'] as $value) {
						$virtualCheck = tep_db_fetch_array(
													tep_db_query("SELECT count(*) AS total
																  FROM	 ".TABLE_PRODUCTS_ATTRIBUTES." pa,
																		 ".TABLE_PRODUCTS_ATTRIBUTES_DOWNLOAD." pad
																  WHERE	 pa.products_id = '".(int)$products_id."'
																  AND	 pa.options_values_id = '".(int)$value."'
																  AND	 pa.products_attributes_id = pad.products_attributes_id")
						);
						if ($virtualCheck['total'] > 0) return true;
					}
					
					for($i = 1; $i < count($productId); ++$i){
						$attributeId = explode('}',$productId[$i]);
						$stringCheckQry = tep_db_query("SELECT	products_options_name
														FROM	".TABLE_PRODUCTS_OPTIONS."
														WHERE	products_options_id = ".(int)$attributeId[0]);
						
						while ($stringCheckRes = tep_db_fetch_array($stringCheckQry)){
							$stringCheck = $stringCheckRes['products_options_name'];
							
							if ($stringCheck == 'downloads' || $stringCheck == 'Downloads' || $stringCheck == 'download' || $stringCheck == 'Download') return true;
						}
					}
				}
			}
		}
		
		//no virtual products found
		return false;
	}
	
	
	/**
	 * check if delivery address exists - doesnt exist, if there are only downloads in cart
	 * @return bool
	 */
	function _deliveryAddressDoesNotExist() {
		global $order;
		
		if (!is_object($order) || !isset($order->delivery) || !is_array($order->delivery)) return true; //delivery address does not exist
		
		$delivery = $order->delivery;
		
		if (!isset($delivery['firstname']) || !isset($delivery['lastname']) || !isset($delivery['company']) || !isset($delivery['street_address']) || !isset($delivery['city']) || !isset($delivery['postcode']) || !isset($delivery['country']['iso_code_2'])) {
			return true; //delivery address does not exist
		}
		
		if (empty($delivery['firstname']) && empty($delivery['lastname']) && empty($delivery['company']) && empty($delivery['street_address']) && empty($delivery['city']) && empty($delivery['postcode']) && empty($delivery['country']['iso_code_2'])) {
			return true; //delivery address does not exist
		}
		
		return false; //delivery address exists
	}
	
	
	/**
	 * Save Shipping-Costs to sofort_products
	 */
	public function insertShippingCostsInSofortTables($ordersId, $shippingItemId) {
		return (tep_db_query("INSERT INTO sofort_products (orders_id, orders_products_id, item_id) VALUES ('".$ordersId."','0','".$shippingItemId."')")) ? true : false;
	}
	
	
	/**
	 * Locates and Saves orders_products_id with corresponding item_id
	 * @param int $ordersId
	 */
	public function insertOrderAttributesInSofortTables($ordersId) {
		$lang = $_SESSION[languages_id];
		$resProd = tep_db_query("SELECT	orders_products_id,
										products_id
								FROM	".TABLE_ORDERS_PRODUCTS."
								WHERE	orders_id = '".HelperFunctions::escapeSql($ordersId)."'");
		
		while($rowProd = tep_db_fetch_array($resProd)){
			$itemId = $rowProd['products_id'];
			$resAttr = tep_db_query("SELECT	products_options,
											products_options_values
									FROM	".TABLE_ORDERS_PRODUCTS_ATTRIBUTES."
									WHERE	orders_id = '".HelperFunctions::escapeSql($ordersId)."'
									AND		orders_products_id = '".HelperFunctions::escapeSql($rowProd['orders_products_id'])."'");
			
			if(mysql_num_rows($resAttr) >= 1) {
				while($rowAttr = tep_db_fetch_array($resAttr)) {
					$resOpt = tep_db_query(" SELECT	products_options_id
											FROM	".TABLE_PRODUCTS_OPTIONS."
											WHERE	products_options_name = '".HelperFunctions::escapeSql($rowAttr['products_options'])."'
											AND		language_id = '".HelperFunctions::escapeSql($lang)."'");
					$rowOpt = tep_db_fetch_array($resOpt);
					$resOptVal = tep_db_query("	SELECT	products_options_values_id
												FROM	".TABLE_PRODUCTS_OPTIONS_VALUES."
												WHERE	products_options_values_name = '".HelperFunctions::escapeSql($rowAttr['products_options_values'])."'
												AND		language_id = '".HelperFunctions::escapeSql($lang)."'");
					
					while($rowOptVal = tep_db_fetch_array($resOptVal)){
						$resCheck = tep_db_query("SELECT	products_options_values_to_products_options_id
												 FROM	".TABLE_PRODUCTS_OPTIONS_VALUES_TO_PRODUCTS_OPTIONS."
												 WHERE	products_options_id = '".HelperFunctions::escapeSql($rowOpt['products_options_id'])."'
												 AND	products_options_values_id = '".HelperFunctions::escapeSql($rowOptVal['products_options_values_id'])."'");
						
						if (mysql_num_rows($resCheck) == 1) $itemId .= "{".$rowOpt['products_options_id']."}".$rowOptVal['products_options_values_id'];
					}
				}
			}
			
			$itemId = HelperFunctions::sortSofortItemId($itemId);
			
			if (!tep_db_query("INSERT INTO sofort_products (orders_id, orders_products_id, item_id) VALUES ('".$ordersId."','".$rowProd['orders_products_id']."','".$itemId."')")) return false;
		}
		
		return true;
	}
	
	
	function _addTax ($amount, $tax) {
		$tax = $tax/100;
		
		return $amount + ($amount * $tax);
	}
}
?>